package com.stu.stusystem.model.system;

import com.stu.stusystem.model.system.serviceInformation.*;
import com.stu.stusystem.util.ArithUtil;
import com.stu.stusystem.util.IPUtil;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.ToString;
import oshi.SystemInfo;
import oshi.hardware.CentralProcessor;
import oshi.hardware.GlobalMemory;
import oshi.hardware.HardwareAbstractionLayer;
import oshi.software.os.FileSystem;
import oshi.software.os.OSFileStore;
import oshi.software.os.OperatingSystem;
import oshi.util.Util;

import java.util.LinkedList;
import java.util.List;
import java.util.Properties;

/**
 * @author: cxt
 * @time: 2021/4/26
 * <p>
 * 服务监控信息
 */
@Data
@ToString
public class ServiceInformation {

    @ApiModelProperty("CPU相关信息")
    private Cpu cpu = new Cpu();

    @ApiModelProperty("內存相关信息")
    private Mem mem = new Mem();

    @ApiModelProperty("JVM相关信息")
    private Jvm jvm = new Jvm();

    @ApiModelProperty("服务器相关信息")
    private Sys sys = new Sys();

    @ApiModelProperty("系统磁盘")
    private List<SysFile> sysFiles = new LinkedList<SysFile>();

    /**
     * 设置服务器信息
     */
    public void setServiceInformation() {
        SystemInfo si = new SystemInfo();
        HardwareAbstractionLayer hal = si.getHardware();
        setCpuInfo(hal.getProcessor());         // 设置CPU信息
        setMemInfo(hal.getMemory());            // 设置内存信息
        setSysInfo();                           // 设置系统信息
        setJvmInfo();                           // 设置JVM信息
        setSysFiles(si.getOperatingSystem());   // 设置磁盘信息
    }

    /**
     * 设置CPU信息
     */
    private void setCpuInfo(CentralProcessor processor) throws NumberFormatException{
        long[] prevTicks = processor.getSystemCpuLoadTicks();
        Util.sleep(100);
        long[] ticks = processor.getSystemCpuLoadTicks();

        long nice = ticks[CentralProcessor.TickType.NICE.getIndex()] - prevTicks[CentralProcessor.TickType.NICE.getIndex()];
        long irq = ticks[CentralProcessor.TickType.IRQ.getIndex()] - prevTicks[CentralProcessor.TickType.IRQ.getIndex()];

        long softIrq = ticks[CentralProcessor.TickType.SOFTIRQ.getIndex()] - prevTicks[CentralProcessor.TickType.SOFTIRQ.getIndex()];
        long steal = ticks[CentralProcessor.TickType.STEAL.getIndex()] - prevTicks[CentralProcessor.TickType.STEAL.getIndex()];
        long cSys = ticks[CentralProcessor.TickType.SYSTEM.getIndex()] - prevTicks[CentralProcessor.TickType.SYSTEM.getIndex()];

        long user = ticks[CentralProcessor.TickType.USER.getIndex()] - prevTicks[CentralProcessor.TickType.USER.getIndex()];
        long ioWait = ticks[CentralProcessor.TickType.IOWAIT.getIndex()] - prevTicks[CentralProcessor.TickType.IOWAIT.getIndex()];
        long idle = ticks[CentralProcessor.TickType.IDLE.getIndex()] - prevTicks[CentralProcessor.TickType.IDLE.getIndex()];
        long totalCpu = user + nice + cSys + idle + ioWait + irq + softIrq + steal;
        int count = processor.getLogicalProcessorCount();
        cpu.setCpuNum(count);       // 设置CPU核心数
        cpu.setTotal(totalCpu);     // 总使用
        cpu.setSys(cSys);           // 系统使用
        cpu.setUsed(user);          // 用户使用
        cpu.setWait(ioWait);        // 用户等待
        cpu.setFree(idle);          // 空闲
    }

    /**
     * 设置内存信息
     */
    private void setMemInfo(GlobalMemory memory) {
        mem.setTotal(memory.getTotal());
        mem.setUsed(memory.getTotal() - memory.getAvailable());
        mem.setFree(memory.getAvailable());
    }

    /**
     * 设置JVM信息
     */
    public void setJvmInfo() {
        Properties props = System.getProperties();
        jvm.setTotal(Runtime.getRuntime().totalMemory());
        jvm.setMax(Runtime.getRuntime().maxMemory());
        jvm.setFree(Runtime.getRuntime().freeMemory());
        jvm.setVersion(props.getProperty("java.version"));
        jvm.setHome(props.getProperty("java.home"));
    }

    /**
     * 设置服务器信息
     */
    public void setSysInfo() {
        Properties props = System.getProperties();
        sys.setComputerName(IPUtil.getHostName());
        sys.setComputerIp(IPUtil.getHostIp());
        sys.setOsName(props.getProperty("os.name"));
        sys.setOsArch(props.getProperty("os.arch"));
        sys.setUserDir(props.getProperty("user.dir"));
    }

    /**
     * 设置磁盘信息
     */
    private void setSysFiles(OperatingSystem os) {
        FileSystem fileSystem = os.getFileSystem();
        List<OSFileStore> fsArray = fileSystem.getFileStores();
        for (OSFileStore fs : fsArray) {
            long free = fs.getUsableSpace();
            long total = fs.getTotalSpace();
            long used = total - free;
            SysFile sysFile = new SysFile();
            sysFile.setDirName(fs.getMount());
            sysFile.setSysTypeName(fs.getType());
            sysFile.setTypeName(fs.getName());
            sysFile.setTotal(convertFileSize(total));
            sysFile.setFree(convertFileSize(free));
            sysFile.setUsed(convertFileSize(used));
            sysFile.setUsage(ArithUtil.mul(ArithUtil.merchant(used, total, 4), 100));
            sysFiles.add(sysFile);
        }
    }

    /**
     * 字节转换
     *
     * @param size 字节大小
     * @return 转换后值
     */
    public String convertFileSize(long size) {
        long kb = 1024;
        long mb = kb * 1024;
        long gb = mb * 1024;
        if (size >= gb) {
            return String.format("%.1f GB", (float) size / gb);
        } else if (size >= mb) {
            float f = (float) size / mb;
            return String.format(f > 100 ? "%.0f MB" : "%.1f MB", f);
        } else if (size >= kb) {
            float f = (float) size / kb;
            return String.format(f > 100 ? "%.0f KB" : "%.1f KB", f);
        } else {
            return String.format("%d B", size);
        }
    }
}
